@@ -15,7 +15,7 @@
 Code for extracting shapes from object.
 """
 import inspect
-from typing import TYPE_CHECKING, Any, Callable, Dict, Sequence, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, Sequence, Tuple, Type
 
 import numpy as np
 import tensorflow as tf
@@ -111,10 +111,23 @@ def get_ndarray_shape(shaped: AnyNDArray, context: ErrorContext) -> Shape:
 @register_get_shape(tf.Tensor)
 @register_get_shape(tf.Variable)
 @register_get_shape(tfp.util.DeferredTensor)
-def get_tensorflow_shape(
-    shaped: Union[tf.Tensor, tf.Variable, tfp.util.DeferredTensor], context: ErrorContext
-) -> Shape:
+def get_tensorflow_shape(shaped: Any, context: ErrorContext) -> Shape:
     shape = shaped.shape
     if not shape:
         return None
     return tuple(shape)
+
+
+@register_get_shape(tfp.python.layers.internal.distribution_tensor_coercible._TensorCoercible)
+def get_tensor_coercible_shape(shaped: Any, context: ErrorContext) -> Shape:
+    # This one is unpleasant. Sometimes the `shape` is a `TensorShape`, but sometimes it's a
+    # function that returns a `TensorShape`. The version of TensorFlow probability seems to have
+    # something to do with it, but it also seems to be more complicated...
+    shape = shaped.shape
+    if not shape:
+        return None
+    if not isinstance(shape, tf.TensorShape):
+        shape = shape()
+        if not shape:
+            return None
+    return tuple(shape)
