@@ -15,7 +15,13 @@
 import numpy as np
 import tensorflow as tf
 
+from ..experimental.check_shapes import check_shapes
 
+
+@check_shapes(
+    "x: [batch...]",
+    "return: [batch...]",
+)
 def inv_probit(x: tf.Tensor) -> tf.Tensor:
     jitter = 1e-3  # ensures output is strictly between 0 and 1
     return 0.5 * (1.0 + tf.math.erf(x / np.sqrt(2.0))) * (1 - 2 * jitter) + jitter
