@@ -16,30 +16,64 @@ import numpy as np
 import tensorflow as tf
 
 from .base import TensorType
+from .experimental.check_shapes import check_shapes
 from .utilities import to_default_float
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "mu: [broadcast shape...]",
+    "var: [broadcast shape...]",
+    "return: [shape...]",
+)
 def gaussian(x: TensorType, mu: TensorType, var: TensorType) -> tf.Tensor:
     return -0.5 * (np.log(2 * np.pi) + tf.math.log(var) + tf.square(mu - x) / var)
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "mu: [broadcast shape...]",
+    "var: [broadcast shape...]",
+    "return: [shape...]",
+)
 def lognormal(x: TensorType, mu: TensorType, var: TensorType) -> tf.Tensor:
     lnx = tf.math.log(x)
     return gaussian(lnx, mu, var) - lnx
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "p: [broadcast shape...]",
+    "return: [shape...]",
+)
 def bernoulli(x: TensorType, p: TensorType) -> tf.Tensor:
     return tf.math.log(tf.where(tf.equal(x, 1), p, 1 - p))
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "lam: [broadcast shape...]",
+    "return: [shape...]",
+)
 def poisson(x: TensorType, lam: TensorType) -> tf.Tensor:
     return x * tf.math.log(lam) - lam - tf.math.lgamma(x + 1.0)
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "scale: [broadcast shape...]",
+    "return: [shape...]",
+)
 def exponential(x: TensorType, scale: TensorType) -> tf.Tensor:
     return -x / scale - tf.math.log(scale)
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "shape: [broadcast shape...]",
+    "scale: [broadcast shape...]",
+    "return: [shape...]",
+)
 def gamma(x: TensorType, shape: TensorType, scale: TensorType) -> tf.Tensor:
     return (
         -shape * tf.math.log(scale)
@@ -49,6 +83,13 @@ def gamma(x: TensorType, shape: TensorType, scale: TensorType) -> tf.Tensor:
     )
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "mean: [broadcast shape...]",
+    "scale: [broadcast shape...]",
+    "df: [broadcast shape...]",
+    "return: [shape...]",
+)
 def student_t(x: TensorType, mean: TensorType, scale: TensorType, df: TensorType) -> tf.Tensor:
     df = to_default_float(df)
     const = (
@@ -61,6 +102,12 @@ def student_t(x: TensorType, mean: TensorType, scale: TensorType, df: TensorType
     )
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "alpha: [broadcast shape...]",
+    "beta: [broadcast shape...]",
+    "return: [shape...]",
+)
 def beta(x: TensorType, alpha: TensorType, beta: TensorType) -> tf.Tensor:
     # need to clip x, since log of 0 is nan...
     x = tf.clip_by_value(x, 1e-6, 1 - 1e-6)
@@ -73,24 +120,30 @@ def beta(x: TensorType, alpha: TensorType, beta: TensorType) -> tf.Tensor:
     )
 
 
+@check_shapes(
+    "x: [broadcast shape...]",
+    "mu: [broadcast shape...]",
+    "sigma: [broadcast shape...]",
+    "return: [shape...]",
+)
 def laplace(x: TensorType, mu: TensorType, sigma: TensorType) -> tf.Tensor:
     return -tf.abs(mu - x) / sigma - tf.math.log(2.0 * sigma)
 
 
+@check_shapes(
+    "x: [D, broadcast N]",
+    "mu: [D, broadcast N]",
+    "L: [D, D]",
+    "return: [N]",
+)
 def multivariate_normal(x: TensorType, mu: TensorType, L: TensorType) -> tf.Tensor:
     """
     Computes the log-density of a multivariate normal.
-    :param x  : Dx1 or DxN sample(s) for which we want the density
-    :param mu : Dx1 or DxN mean(s) of the normal distribution
-    :param L  : DxD Cholesky decomposition of the covariance matrix
-    :return p : (1,) or (N,) vector of log densities for each of the N x's and/or mu's
 
-    x and mu are either vectors or matrices. If both are vectors (N,1):
-    p[0] = log pdf(x) where x ~ N(mu, LL^T)
-    If at least one is a matrix, we assume independence over the *columns*:
-    the number of rows must match the size of L. Broadcasting behaviour:
-    p[n] = log pdf of:
-    x[n] ~ N(mu, LL^T) or x ~ N(mu[n], LL^T) or x[n] ~ N(mu[n], LL^T)
+    :param x: sample(s) for which we want the density
+    :param mu: mean(s) of the normal distribution
+    :param L: Cholesky decomposition of the covariance matrix
+    :return: log densities
     """
 
     d = x - mu
@@ -100,11 +153,4 @@ def multivariate_normal(x: TensorType, mu: TensorType, L: TensorType) -> tf.Tens
     p -= 0.5 * num_dims * np.log(2 * np.pi)
     p -= tf.reduce_sum(tf.math.log(tf.linalg.diag_part(L)))
 
-    shape_constraints = [
-        (d, ["D", "N"]),
-        (L, ["D", "D"]),
-        (p, ["N"]),
-    ]
-    tf.debugging.assert_shapes(shape_constraints, message="multivariate_normal()")
-
     return p
