@@ -523,7 +523,10 @@ class GPRFITC(SGPRBase_deprecated):
 
 class SGPR_with_posterior(SGPR_deprecated):
     """
-    This is an implementation of GPR that provides a posterior() method that
+    Sparse Variational GP regression.
+    The key reference is :cite:t:`titsias2009variational`.
+
+    This is an implementation of SGPR that provides a posterior() method that
     enables caching for faster subsequent predictions.
     """
 
