@@ -16,6 +16,7 @@ from typing import Any
 import numpy as np
 import pytest
 import tensorflow as tf
+import tensorflow_probability as tfp
 
 from gpflow.base import Parameter
 from gpflow.experimental.check_shapes import Shape, get_shape
@@ -24,6 +25,19 @@ from gpflow.experimental.check_shapes.exceptions import NoShapeError
 from .utils import TestContext
 
 
+def make_tensor_coercible(
+    shape: Shape, concrete: bool
+) -> tfp.python.layers.internal.distribution_tensor_coercible._TensorCoercible:
+    loc = tf.zeros(shape)
+    scale = tf.ones(shape)
+    dist = tfp.python.layers.internal.distribution_tensor_coercible._TensorCoercible(
+        tfp.distributions.Normal(loc, scale), lambda self: loc
+    )
+    if concrete:
+        tf.convert_to_tensor(dist)  # Triggers some caching within `dist`.
+    return dist
+
+
 @pytest.mark.parametrize(
     "shaped,expected_shape",
     [
@@ -46,6 +60,10 @@ from .utils import TestContext
         (tf.Variable(np.zeros((2, 4)), shape=tf.TensorShape(None)), None),
         (Parameter(3), ()),
         (Parameter(np.zeros((4, 2))), (4, 2)),
+        (make_tensor_coercible((), True), ()),
+        (make_tensor_coercible((4, 5), True), (4, 5)),
+        (make_tensor_coercible((), False), None),
+        (make_tensor_coercible((4, 5), False), None),
     ],
 )
 def test_get_shape(shaped: Any, expected_shape: Shape) -> None:
